@@ -1,15 +1,19 @@
-## ----include=FALSE-------------------------------------------------------
-options(clustermq.scheduler = 'local')
-library(clustermq)
+## ----echo=FALSE, results="hide"------------------------------------------
+options(clustermq.scheduler = "local")
+knitr::opts_chunk$set(
+    cache = FALSE,
+    echo = TRUE,
+    collapse = TRUE,
+    comment = "#>"
+)
 
-## ----eval=FALSE----------------------------------------------------------
-#  # load the library and create a simple function
-#  library(clustermq)
-#  fx = function(x) x * 2
-#  
-#  # queue the function call on your scheduler
-#  Q(fx, x=1:3, n_jobs=1)
-#  # list(2,4,6)
+## ------------------------------------------------------------------------
+# load the library and create a simple function
+library(clustermq)
+fx = function(x) x * 2
+
+# queue the function call on your scheduler
+Q(fx, x=1:3, n_jobs=1)
 
 ## ----eval=FALSE----------------------------------------------------------
 #  install.packages('clustermq')
@@ -33,10 +37,15 @@ Q(fx, x=1:3, export=list(y=10), n_jobs=1)
 
 ## ------------------------------------------------------------------------
 fx = function(x) {
-    library(dplyr)
+    `%>%` = dplyr::`%>%`
     x %>%
-        mutate(area = Sepal.Length * Sepal.Width) %>%
+        dplyr::mutate(area = Sepal.Length * Sepal.Width) %>%
         head()
 }
 Q(fx, x=list(iris), n_jobs=1)
 
+## ------------------------------------------------------------------------
+library(foreach)
+register_dopar_cmq(n_jobs=2, memory=1024) # accepts same arguments as `workers`
+foreach(i=1:3) %dopar% sqrt(i) # this will be executed as jobs
+
